﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Interop.Plantronics;
using System.Runtime.InteropServices;

namespace SpokesSDKCOMNETSample
{
    class DeviceEventsListener : EventListener
    {
        private COMDevice device = null;
        private ICOMDeviceEvents_Event deviceEvents = null;
        private ICOMDeviceListenerEvents_Event deviceListenerEvents = null;
        
        public DeviceEventsListener(COMSession session)
        {
            try
            {
                device = session.GetActiveDevice();
                if (device != null)
                {
                    HookToDeviceEvents();
                    HookToDeviceListenerEvents();
                    PrintDeviceInfo();
                }
                else
                    Console.WriteLine("Unable to retrieve active device");
            }
            catch (System.Exception ex)
            {
                Console.WriteLine("Unable to retrieve/hook to active device");
            }
        }

        ~DeviceEventsListener()
        {
            if (device != null)
            {
                Marshal.ReleaseComObject(device);
                device = null;
            }
            if (deviceEvents != null)
            {
                deviceEvents.onAudioStateChanged -= onAudioStateChanged;
                deviceEvents.onButtonPressed -= onButtonPressed;
                deviceEvents.onFlashButtonPressed -= onFlashButtonPressed;
                deviceEvents.onMuteStateChanged -= onMuteStateChanged;
                deviceEvents.onSmartButtonPressed -= onSmartButtonPressed;
                deviceEvents.onTalkButtonPressed -= onTalkButtonPressed;
                deviceEvents.onDataReceived -= onDataReceived;

                Marshal.ReleaseComObject(deviceEvents);
                deviceEvents = null;
            }
            if (deviceListenerEvents != null)
            {
                deviceListenerEvents.onATDStateChanged -= deviceListenerEventHandler;
                deviceListenerEvents.onBaseButtonPressed -= deviceListenerEventHandler;
                deviceListenerEvents.onBaseStateChanged -= deviceListenerEventHandler;
                deviceListenerEvents.onHeadsetButtonPressed -= deviceListenerEventHandler;
                deviceListenerEvents.onHeadsetStateChanged -= deviceListenerEventHandler;

                Marshal.ReleaseComObject(deviceListenerEvents);
                deviceListenerEvents = null;
            }
        }

        private void HookToDeviceEvents()
        {
            deviceEvents = device as ICOMDeviceEvents_Event;
            if (deviceEvents != null)
            {
                deviceEvents.onAudioStateChanged += onAudioStateChanged;
                deviceEvents.onButtonPressed += onButtonPressed;
                deviceEvents.onFlashButtonPressed += onFlashButtonPressed;
                deviceEvents.onMuteStateChanged += onMuteStateChanged;
                deviceEvents.onSmartButtonPressed += onSmartButtonPressed;
                deviceEvents.onTalkButtonPressed += onTalkButtonPressed;
                deviceEvents.onDataReceived += onDataReceived;

                Console.WriteLine("Successfully hooked to device events");

                ICOMDeviceEventsExt_Event deviceEventsExt = device as ICOMDeviceEventsExt_Event;
                if (deviceEventsExt != null)
                {
                    deviceEventsExt.onBatteryLevelChanged += new ICOMDeviceEventsExt_onBatteryLevelChangedEventHandler(deviceEventsExt_onBatteryLevelChanged);
                    deviceEventsExt.onHeadsetStateChanged += new ICOMDeviceEventsExt_onHeadsetStateChangedEventHandler(deviceEventsExt_onHeadsetStateChanged);                    
                }

                ICOMBaseEvents_Event baseEvents = device as ICOMBaseEvents_Event;
                if (baseEvents != null)
                {
                    baseEvents.onBaseEventReceived += new ICOMBaseEvents_onBaseEventReceivedEventHandler(baseEvents_onBaseEventReceived);
                }

                ICOMMobilePresenceEvents_Event mobilePresenceEvents = device as ICOMMobilePresenceEvents_Event;
                if (mobilePresenceEvents != null)
                {
                    mobilePresenceEvents.onPresenceChanged += new ICOMMobilePresenceEvents_onPresenceChangedEventHandler(mobilePresenceEvents_onPresenceChanged);
                }
            }
            else
                Console.WriteLine("Unable to hook to device events");
        }

        private void HookToDeviceListenerEvents()
        {
            deviceListenerEvents = device.DeviceListener as ICOMDeviceListenerEvents_Event;
            if (deviceListenerEvents != null)
            {
                deviceListenerEvents.onATDStateChanged += deviceListenerEventHandler;
                deviceListenerEvents.onBaseButtonPressed += deviceListenerEventHandler;
                deviceListenerEvents.onBaseStateChanged += deviceListenerEventHandler;
                deviceListenerEvents.onHeadsetButtonPressed += deviceListenerEventHandler;
                deviceListenerEvents.onHeadsetStateChanged += deviceListenerEventHandler;

                Console.WriteLine("Successfully hooked to device listener events");
            }
            else
                Console.WriteLine("Unable to hook to device listener events");
        }

        private void PrintDeviceInfo()
        {
            if (device == null)
                return;

            string usbFirmware = "";
            string bluethoothFirmware = "";
            string baseFirmware = "";
            string remoteFirmware = "";
            
            try
            {
                usbFirmware = device.HostCommand.GetFirmwareVersion(FirmwareVersionType.VersionType_USBFirmware);
            }
            catch (COMException)
            {
            }

            try
            {
                bluethoothFirmware = device.HostCommand.GetFirmwareVersion(FirmwareVersionType.VersionType_BluetoothFirmware);
            }
            catch (COMException)
            {
            }

            try
            {
                baseFirmware = device.HostCommand.GetFirmwareVersion(FirmwareVersionType.VersionType_BaseFirmware);
            }
            catch (COMException)
            {
            }

            try
            {
                remoteFirmware = device.HostCommand.GetFirmwareVersion(FirmwareVersionType.VersionType_RemoteFirmware);
            }
            catch (COMException)
            {            	
            }
            
            Console.WriteLine("********************* Device Information ******************** ");
            Console.WriteLine("Internal Name:       " + device.InternalName);
            Console.WriteLine("Manufacturer Name:   " + device.ManufacturerName);
            Console.WriteLine("Product Name:        " + device.ProductName);
            Console.WriteLine(string.Format("Vendor ID:           0x{0:X}", device.VendorId));
            Console.WriteLine(string.Format("Product ID:          0x{0}", device.ProductId.ToString("X").PadLeft(4, '0')));
            Console.WriteLine("Version Num:         " + device.VersionNumber);
            Console.WriteLine("USB firmware:        " + (string.IsNullOrEmpty(usbFirmware) ? "none" : usbFirmware));
            Console.WriteLine("Base firmware:       " + (string.IsNullOrEmpty(baseFirmware) ? "none" : baseFirmware));
            Console.WriteLine("Remote firmware:     " + (string.IsNullOrEmpty(remoteFirmware) ? "none" : remoteFirmware));
            Console.WriteLine("Bluetooth firmware:  " + (string.IsNullOrEmpty(bluethoothFirmware) ? "none" : bluethoothFirmware));
            Console.WriteLine("************************************************************* ");
        }

        //DeviceEvents
        private void onDataReceived(ref object report)
        {
            WriteOutput("DeviceEvents: buffer received " + ByteArrayToString( report as Byte[] ));
        }
        private void onAudioStateChanged(COMDeviceEventArgs args)
        {
            WriteOutput("DeviceEvents: audio state changed: " + args.AudioState.ToString());
        }

        private void onButtonPressed(COMDeviceEventArgs args)
        {
            WriteOutput("DeviceEvents: button pressed: " + args.ButtonPressed.ToString());
        }

        private void onFlashButtonPressed(COMDeviceEventArgs args)
        {
            WriteOutput("DeviceEvents: flash button pressed");
        }

        private void onMuteStateChanged(COMDeviceEventArgs args)
        {
            WriteOutput("DeviceEvents: mute state changed: " + args.mute);
        }

        private void onSmartButtonPressed(COMDeviceEventArgs args)
        {
            WriteOutput("DeviceEvents: smart button pressed");
        }

        private void onTalkButtonPressed(COMDeviceEventArgs args)
        {
            WriteOutput("DeviceEvents: talk button pressed");
        }

        // base events
        void baseEvents_onBaseEventReceived(COMBaseEventArgs args)
        {
            WriteOutput("BaseEvent: base event: " + args.EventType.ToString());
        }

        // device ext events
        void deviceEventsExt_onBatteryLevelChanged(COMBatteryLevelEventArgs args)
        {
            WriteOutput("DeviceEventsExt: battery level changed: " + args.BatteryLevel.ToString());
        }

        void deviceEventsExt_onHeadsetStateChanged(COMHeadsetStateEventArgs args)
        {
            WriteOutput("DeviceEventsExt: headset state changed: " + args.HeadsetState.ToString());
        }

        // mobile presence evetns
        void mobilePresenceEvents_onPresenceChanged(COMMobilePresenceEventArgs args)
        {
            WriteOutput("MobilePresenceEvents: presence changed: " + args.MobileCallState.ToString());
        }

        //DeviceListener events
        private void deviceListenerEventHandler(COMDeviceListenerEventArgs args)
        {
            switch (args.DeviceEventType)
            {
                case COMDeviceEventType.DeviceEventType_ATDStateChanged:
                    WriteOutput("DeviceListener: ATD state changed: {0}", args.ATDStateChange.ToString());
                    break;
                case COMDeviceEventType.DeviceEventType_BaseButtonPressed:
                    if (args.BaseButton == DeviceBaseButton.BaseButton_DialPad)
                    {
                        WriteOutput("DeviceListener: base button: {0}, dialed key: {1}", args.BaseButton, args.DialedKey);
                    }
                    else
                    {
                        WriteOutput("DeviceListener: base button: {0}", args.BaseButton);
                    }
                    break;
                case COMDeviceEventType.DeviceEventType_BaseStateChanged:
                    WriteOutput("DeviceListener: base button: {0}, base state: {1}", args.BaseButton.ToString(), args.BaseStateChange.ToString());
                    break;
                case COMDeviceEventType.DeviceEventType_HeadsetButtonPressed:
                case COMDeviceEventType.DeviceEventType_HeadsetStateChanged:
                    WriteOutput("DeviceListener: headset button: {0}, headset state: {1}", args.HeadsetButton.ToString(), args.HeadsetStateChange.ToString());
                    break;
                default:
                    WriteOutput("DeviceListener event");
                    break;
            }
        }
        public static string ByteArrayToString(byte[] ba)
        {
            if (ba == null) return "";
            StringBuilder hex = new StringBuilder(ba.Length * 2);
            foreach (byte b in ba) hex.AppendFormat("{0:x2}", b);
            return hex.ToString();
        }
    }
}
